@@ -19,14 +19,16 @@
 
 
 import numpy as np
-import pandas as pd
 from sklearn.preprocessing import MinMaxScaler
 from sklearn.metrics import mean_squared_error
+from sklearn.metrics import make_scorer
+from sklearn.model_selection import TimeSeriesSplit
+from sklearn.model_selection import GridSearchCV
 
 from matplotlib import pyplot as plt
-plt.rcParams['image.cmap'] = 'jet'
-plt.rcParams['pdf.fonttype'] = 42
-plt.rcParams['ps.fonttype'] = 42
+plt.rcParams["font.family"] = "Times New Roman"
+plt.rcParams["font.size"] = 10
+
 
 from pyrcn.echo_state_network import ESNRegressor
 from pyrcn.linear_model import IncrementalRegression
@@ -45,8 +47,19 @@ from pyrcn.base import InputToNode, NodeToNode
 # In[2]:
 
 
-time_series = pd.read_csv(filepath_or_buffer="examples/dataset/GC=F.csv")
-time_series.head()
+X = np.genfromtxt(fname="./examples/dataset/GC=F.csv", usecols=4, skip_header=1,delimiter=",")
+
+unit_impulse = np.zeros(shape=(100, 1), dtype=int)
+unit_impulse[5] = 1
+
+plt.figure()
+plt.plot(X)
+plt.xlim([0, 100])
+plt.ylim([0, 1])
+plt.xlabel('n')
+plt.ylabel('X[n]')
+plt.grid()
+plt.show()
 
 
 # Analyzing the dataset, we can analyze the structure. The first column is the date, the second column is the open value,
@@ -59,9 +72,7 @@ time_series.head()
 # In[3]:
 
 
-time_series = time_series.drop(columns=["Open", "High", "Low", "Adj Close", "Volume"], axis=1)
-time_series = time_series.dropna()
-time_series.head()
+X = X[~np.isnan(X)].reshape(-1, 1)
 
 
 # As one can see, the stock price trends upwards for the first few thousand days. Then goes down at around $n=3300$ before climbing again after around $n = 4900$. 
@@ -69,10 +80,13 @@ time_series.head()
 # In[4]:
 
 
-plt.figure()
-time_series.plot(x='Date', y='Close', legend=False, grid=True)
+plt.figure(figsize=(4, 2.5))
+plt.plot(X)
 plt.xlabel("Timestamp")
+plt.xlim([0, len(X)])
 plt.ylabel("Price")
+plt.grid()
+plt.tight_layout()
 plt.show()
 
 
@@ -81,22 +95,10 @@ plt.show()
 # In[5]:
 
 
-prices = np.asarray(time_series['Close'])
-
-scaler = MinMaxScaler()
-prices = scaler.fit_transform(X=prices.reshape(-1, 1))
-
-
-# Define Train/Test lengths:
-# 
-# The first 3000 values were used to train the ESN and to optimize the hyper-parameters and the rest for testing. 
-
-# In[6]:
-
-
 train_len = 3000
 future_len = 1
-future_total = len(prices) - train_len
+
+scaler = MinMaxScaler(feature_range=(-1, 1)).fit(X=X)
 
 
 # Echo State Network preparation
@@ -116,18 +118,65 @@ esn = ESNRegressor(input_to_node=base_input_to_nodes,
 
 # In[8]:
 
+X_train = scaler.transform(X[0:train_len])
+y_train = scaler.transform(X[1:train_len+1])
+X_test = scaler.transform(X[train_len+1:-1])
+y_test = scaler.transform(X[train_len+1+future_len:])
 
-train_in = prices[0:train_len, :]
-train_out = prices[0+1:train_len+1, :]
-test_in = prices[0:train_len+future_total - future_len, :]
-test_out = prices[future_len:train_len+future_total, :]
+fig = plt.figure()
+plt.plot(scaler.transform(X.reshape(-1, 1)))
+plt.xlabel("n")
+plt.xlim([0, len(X)])
+plt.ylabel("u[n]")
+plt.grid()
+fig.set_size_inches(4, 2.5)
+plt.savefig('input_data.pdf', bbox_inches = 'tight', pad_inches = 0)
 
-esn.fit(X=train_in, y=train_out.ravel())
-train_pred = esn.predict(X=train_in)
-test_pred = esn.predict(X=test_in)
+param_grid = {'input_to_node__hidden_layer_size': [100],
+              'input_to_node__input_scaling': [0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0,  1.1, 1.2, 1.3, 1.4, 1.5],
+              'input_to_node__bias_scaling': [0.0],
+              'input_to_node__activation': ['identity'],
+              'input_to_node__random_state': [42],
+              'node_to_node__hidden_layer_size': [100],
+              'node_to_node__leakage': [1.0],
+              'node_to_node__spectral_radius': [0.0, 0.1, 0.2, 0.3, 0.4, 0.5, 0.6, 0.7, 0.8, 0.9, 1.0, 1.1, 1.2, 1.3, 1.4, 1.5],
+              'node_to_node__bias_scaling': [0.0],
+              'node_to_node__bi_directional': [False],
+              'node_to_node__continuation': [True],
+              'node_to_node__activation': ['tanh'],
+              'node_to_node__wash_out': [0],
+              'node_to_node__random_state': [42],
+              'regressor__alpha': [1e-5],
+              'random_state': [42] }
 
-train_err = mean_squared_error(y_true=train_out, y_pred=train_pred)
-test_err = mean_squared_error(y_true=test_out[train_len+1:, :], y_pred=test_pred[train_len+1:])
+scorer = make_scorer(score_func=mean_squared_error, greater_is_better=False)
+
+ts_split = TimeSeriesSplit()
+grid_search = GridSearchCV(ESNRegressor(), cv=ts_split, param_grid=param_grid, scoring=scorer, n_jobs=-1).fit(X=X_train, y=y_train)
+
+print(grid_search.best_params_)
+
+esn = grid_search.best_estimator_
+# esn.set_params(**{'node_to_node__leakage': 0.1})
+# esn.fit(X=X_train, y=y_train)
+esn.predict(X=unit_impulse)
+fig = plt.figure()
+im = plt.imshow(np.abs(esn._node_to_node._hidden_layer_state[:, 1:].T),vmin=0, vmax=1)
+plt.xlim([0, 100])
+plt.ylim([0, esn._node_to_node._hidden_layer_state[:, 1:].shape[1] - 1])
+plt.xlabel('n')
+plt.ylabel('R[n]')
+plt.colorbar(im)
+plt.grid()
+fig.set_size_inches(4, 2.5)
+plt.savefig('InputScaling_SpectralRadius.pdf', bbox_inches = 'tight', pad_inches = 0)
+
+# esn.fit(X=X_train, y=y_train.ravel())
+y_train_pred = esn.predict(X=X_train)
+y_test_pred = esn.predict(X=X_test)
+
+train_err = mean_squared_error(y_true=y_train, y_pred=y_train_pred)
+test_err = mean_squared_error(y_true=y_test, y_pred=y_test_pred)
 
 print("Train MSE:\t{0}".format(train_err))
 print("Test MSE:\t{0}".format(test_err))
@@ -137,14 +186,9 @@ print("Test MSE:\t{0}".format(test_err))
 
 # In[9]:
 
-
-prices_pred = np.empty_like(prices)
-prices_pred[:] = np.nan
-prices_pred[train_len + future_len:, 0] = test_pred[train_len:]
-
 plt.figure()
-plt.plot(scaler.inverse_transform(test_out), label='Target')
-plt.plot(scaler.inverse_transform(prices_pred), label='Predicted')
+plt.plot(scaler.inverse_transform(y_test), label='Target')
+plt.plot(scaler.inverse_transform(y_test_pred.reshape(-1, 1)), label='Predicted')
 plt.xlabel("Timestamp")
 plt.ylabel("Price")
 plt.legend()
